"""
Mixin for state containers to maintain information neccessary to compute
cumulative means and standard deviations, and the means to do so.
"""

from math import sqrt

class CumulativeStandardDeviation:
    """
    Mixin for state containers to maintain information neccessary to compute
    cumulative means and standard deviations, and the means to do so.
    """

    nobs = 0
    todays_returns = None
    total_returns = 0
    total_returns_squared = 0

    @property
    def mean_and_standard_deviation(self):
        """
        Compute and return a tuple representing the current mean and
        standard deviation.
        """

        nobs = self.nobs
        mean = self.total_returns / nobs
        standard_deviation = sqrt(
            self.total_returns_squared / nobs -
            mean * mean
        )

        return mean, standard_deviation

    def process_todays_returns(self, todays_returns):
        """Update the state with the computed returns"""

        self.todays_returns = todays_returns
        if todays_returns is not None:
            self.nobs += 1
            self.total_returns += todays_returns
            self.total_returns_squared += todays_returns * todays_returns

    def copy_from(self, other):
        self.nobs = other.nobs
        self.todays_returns = other.todays_returns
        self.total_returns = other.total_returns
        self.total_returns_squared = other.total_returns_squared

