from functools import cached_property

from ..cumulative_standard_deviation import CumulativeStandardDeviation
from .revision_state import RevisionState


class PortfolioState(CumulativeStandardDeviation):
    """Portfolio state container for the portfolio result computer."""

    current_revision_state = None

    def __init__(self, portfolio):
        self._portfolio = portfolio

    @property
    def key(self):
        """By what to identify this portfolio in the result output"""

        return self._portfolio.pk

    @cached_property
    def revision_states(self):
        """Initializer for the list of revisions of this portfolio state"""

        return [
            RevisionState(revision)
            for revision in self._sorted_revisions
        ]

    @property
    def _sorted_revisions(self):
        return sorted(
            self._revisions,
            key=lambda revision: revision.date
        )

    @property
    def _revisions(self):
        return self._portfolio.portfoliorevision_set.all()
