"""Asset related tests for the stocks module"""

from json import loads as parse_json

from django.test import TestCase
from rest_framework.test import APIClient


class TestAssets(TestCase):
    """Asset related tests"""

    fixtures = ['stocks']
    client_class = APIClient

    def test_list(self):
        """Fetching all assets"""

        response = self.client.get('/assets')
        self.assertEqual(response.status_code, 200)
        content = parse_json(response.content)
        self.assertEqual(len(content), 10)

    def test_create(self):
        """Creating assets should be impossible."""

        response = self.client.post('/assets', {
            'isin': '0123456789AB',
            'name': 'Test asset',
            'ticker': 'TEST',
        }, format='json')
        self.assertEqual(
            response.status_code,
            403,
            "Assets cannot be created",
        )

    def test_update(self):
        """Changing assets should be impossible."""

        response = self.client.put('/assets/1', {
            'isin': '0123456789AB',
            'name': 'Test asset',
            'ticker': 'TEST',
        }, format='json')
        self.assertEqual(
            response.status_code,
            403,
            "Assets cannot be updated",
        )

    def test_delete(self):
        """Deleting assets should be impossible."""

        response = self.client.delete('/assets/1')
        self.assertEqual(
            response.status_code,
            403,
            "Assets cannot be updated",
        )
