"""Index related tests for the stocks module"""

from json import loads as parse_json

from django.test import TestCase
from rest_framework.test import APIClient


class TestIndexes(TestCase):
    """Index related tests"""

    fixtures = ['stocks']
    client_class = APIClient

    def test_list(self):
        """Fetching all indexes"""

        response = self.client.get('/indexes')
        self.assertEqual(response.status_code, 200)
        content = parse_json(response.content)
        self.assertEqual(len(content), 3)

    def test_create(self):
        """Creating indexes should be impossible."""

        response = self.client.post('/indexes', {
            'name': 'Test index',
        }, format='json')
        self.assertEqual(
            response.status_code,
            403,
            "Indexes cannot be created",
        )

    def test_update(self):
        """Changing indexes should be impossible."""

        response = self.client.put('/indexes/1', {
            'name': 'Test index',
        }, format='json')
        self.assertEqual(
            response.status_code,
            403,
            "Indexes cannot be updated",
        )

    def test_delete(self):
        """Deleting indexes should be impossible."""

        response = self.client.delete('/indexes/1')
        self.assertEqual(
            response.status_code,
            403,
            "Indexes cannot be updated",
        )

    def test_assets(self):
        """Fetching assets for an index"""

        response = self.client.get('/indexes/1/assets')
        self.assertEqual(response.status_code, 200)
        content = parse_json(response.content)
        self.assertEqual(len(content), 7)
