#! /usr/bin/python3

import json
import ldap.filter
import investment_game.settings


def main():
    # There are 6 initial admin users, so skip those pks
    sequences = {'uvt.User': 6}

    def add_fixture(model, fields):
        pk = sequences.get(model, 0) + 1
        sequences[model] = pk
        # Add model first so the order doesn't change too much:
        fixtures.append({'model': model, 'pk': pk, 'fields': fields})
        return pk

    UVT_USER_BACKEND = investment_game.settings.UVT_USER_BACKEND

    connection = ldap.initialize(UVT_USER_BACKEND['ldap_server_uri'])
    connection.set_option(ldap.OPT_NETWORK_TIMEOUT, 5)
    connection.set_option(ldap.OPT_X_TLS_CACERTDIR, '/etc/ssl/certs')
    connection.set_option(
        ldap.OPT_X_TLS_CACERTFILE, '/etc/ssl/certs/ca-certificates.crt'
    )
    connection.set_option(ldap.OPT_X_TLS_REQUIRE_CERT, ldap.OPT_X_TLS_DEMAND)
    # apply the SSL configuration
    connection.set_option(ldap.OPT_X_TLS_NEWCTX, 0)
    connection.start_tls_s()

    uids = [f'u{n:06}' for n in range(900, 929)]

    fixtures = []

    search_filter = ''.join(
        ldap.filter.filter_format('(uid=%s)', (uid,))
        for uid in uids
    )
    attributes = {'uid': 'username', **UVT_USER_BACKEND['attributes']}
    results = connection.search_s(
        UVT_USER_BACKEND['search_base'],
        ldap.SCOPE_SUBTREE,
        f'(|{search_filter})',
        list(attributes),
    )

    result_fields = {}
    for result in results:
        _, result_attrs = result
        fields = {
            value: str(result_attrs[key][0], 'UTF-8', 'ignore')
            for key, value in attributes.items()
            if key in result_attrs
        }
        if 'full_name' not in fields:
            try:
                fields['full_name'] = (
                    f"{fields['first_name']} {fields['last_name']}"
                )
            except KeyError:
                pass
        result_fields[fields['username']] = fields

    for uid in uids:
        add_fixture('uvt.User', result_fields[uid])

    sorted_fixtures = sorted(
        fixtures,
        key=lambda elem: (elem['model'], elem['pk']),
    )

    with open(f'investment_game/uvt/fixtures/players.json', 'w') as fh:
        json.dump(sorted_fixtures, fh, indent=2)


if __name__ == '__main__':
    main()
